;
; STOCK 810 ROM
; -------------
;
; Automata states (Used in LAUTOM).
; Each bit has a different meaning and is used to know what should be done next.
;
AUTLINE  = $01 ; the Atari computer has raised the command line
AUTBAD   = $02 ; an unknown command has been received
AUTWAIT  = $04 ; a FDC operation is in progress. Wait before sending a new command
AUTVER   = $08 ; is it a write with verify ?
AUTTRK0  = $20 ; the head has been restored to track 0
AUT40    = $40 ; never set but checked once (related to SIO command line)
AUTCMD   = $80 ; stable SIO command line state (command line is up during a certain period of time)
;
; Command Status bits (Used in LSTATCMD).
;
STATNONE = $00
STATCMD  = $01
STATDATA = $02
STATERR  = $04
STATWP   = $08
STATACT  = $10
;
; Controller Status bits (Used in LSTATHW).
;
CTRLBUSY = $01
CTRLDRQ  = $02
CTRLLOST = $04
CTRLCRC  = $08
CTRLRNF  = $10
CTRLTYPE = $20
CTRLWP   = $40
CTRLMON  = $80
;
; Controller commands.
; These values are written (inverted) into FCOMMAND register to ask the controller for a particular task.
;
FDCWTRK = $F4
FDCFINT = $D0
FDCWSEC = $A8
FDCRSEC = $88
FDCSEEK = $13
FDCREST = $00
;
; Special bytes for FDC
; These values are written (inverted) into FDATA register to write address marks or CRC
;
FDCIDAM  = $FE
FDCIDXAM = $FC
FDCDATAM = $FB
FDCCRC   = $F7
;
; Length of Status answer
; Length of checksum
; Timeout for the Format command
;
STATLEN  = $04
SUMLEN   = $01
TIMEOUT  = $E0
;
; Answer to a command.
;
ACK      = $41 ; 'A'
COMPLETE = $43 ; 'C'
ERROR    = $45 ; 'E'
NACK     = $4E ; 'N'
;
; FDC 1771 registers
;
FSTATUS  = $00 ; Read
FCOMMAND = $00 ; Write
FTRACK   = $01
FSECTOR  = $02
FDATA    = $03
;
; WDC base address
;
BASEFDC  = $0000
;
; WDC addresses
;
FDCSTAT  = BASEFDC+FSTATUS
FDCCOMND = BASEFDC+FCOMMAND
FDCTRACK = BASEFDC+FTRACK
FDCSECT  = BASEFDC+FSECTOR
FDCDATA  = BASEFDC+FDATA
;
; RIOT 6532 registers
;
DRA      = $00
DDRA     = $01
DRB      = $02
DDRB     = $03
EDGECTRL = $04
RTIMNOIT = $14
RINTFLAG = $15
RTIMIT   = $1C
WR64IT   = $1E
WR1024IT = $1F
;
; Port A bits
; Read drive number, motor ON flag and information from FDC
;
SWITCH1  = $01
DRVMOTOR = $02
SWITCH2  = $04
UNUSED   = $08
WPT1771  = $10
VSS6532  = $20
IRQ1771  = $40
DRQ1771  = $80
;
; Port B bits
; Read and write to SIO, and stepper motor thru phases.
;
DATAOUT  = $01
VCCRDY   = $02
PHASE4   = $04
PHASE3   = $08
PHASE2   = $10
PHASE1   = $20
COMMAND  = $40
DATAIN   = $80
;
; RIOT base address
;
BASERIOT = $0380
;
; RIOT addresses
;
RIOTDRA  = BASERIOT+DRA
RIOTDDRA = BASERIOT+DDRA
RIOTDRB  = BASERIOT+DRB
RIOTDDRB = BASERIOT+DDRB
RIOTEDGE = BASERIOT+EDGECTRL
RIOTRTNO = BASERIOT+RTIMNOIT
RIOTRINT = BASERIOT+RINTFLAG
RIOTRTIT = BASERIOT+RTIMIT
RIOTW64  = BASERIOT+WR64IT
RIOTW1K  = BASERIOT+WR1024IT
;
; Sector buffer
;
SECTOR   = $0080
;
; Sector relative variables
;
LSECPTR  = SECTOR+$60
LISECTOR = SECTOR+$62
LITRACK  = SECTOR+$63
;
; Ram variables
;
LCMDDRV  = $0181
LCMDCMD  = $0182
LCMDAUX1 = $0183
LCMDAUX2 = $0184
LSNDBYTE = $0186
LCRCBYTE = $0187
LCURDRB  = $01CD
LCURTRK  = $01CE
LTRDIF   = $01CF
LSTATCMD = $01D0
LSTATHW  = $01D1
LSTATTIM = $01D2
LSTATSUM = $01D4
LSAVAUX2 = $01D7
LSAVAUX1 = $01D8
LITRKNUM = $01DA
LISECNUM = $01DB
LAUTOM   = $01DD
LENDCMD  = $01DE
LPAUSE   = $01DF
LRETRIES = $01E0
LOFSRES  = $01E1
LSECNUM  = $01E2
LTRKTEMP = $01E3
LABSECHI = $01E4
LJMPCMD  = $01E8
LOFSCMD  = $01EA
LNBIRQ   = $01EB
;
; Start of code
;
           OPT h-
           ORG $0800
;
; Code starts here when the drive is turned on.
;
START      CLD
;
; Configure DRVMOTOR and UNUSED pins as output.
; All other pins (SWITCH1, SWITCH2, WPT1771,...) are input pins.
;
           LDA #DRVMOTOR+UNUSED
           STA RIOTDRA
           STA RIOTDDRA
;
; Configure PHASE1-4 pins as output.
; All other pins are input pins even DATAOUT so that no data is sent on the SIO bus now.
;
           LDX #PHASE1+PHASE2+PHASE3+PHASE4
           STX RIOTDDRB
           STX RIOTEDGE
           LDX #0
           STX RIOTDRB
;
; Stop current controller operation (if any) by sending a Force Interrupt command.
; This will fill the status register.
;
           LDA #$FF-FDCFINT
           STA FDCCOMND
;
; Remember to wait so that Force Interrupt command is not cancelled.
;
           LDA #AUTWAIT+AUTLINE
           STA LAUTOM
;
; Fill last part of status buffer returned on a GET STATUS command.
;
           LDA #TIMEOUT
           STA LSTATTIM
           LDA #0
           STA LSTATTIM+1
;
; Sleep to let Force Interrupt command terminate.
;
           JSR LCHKWAIT
;
; Step to track 39
;
           LDY #$27
           JSR LMOVHEAD
;
; Set track number to 0 ($FF inverted) and sector number to 1 ($FE inverted)
;
           STY FDCTRACK
           DEY
           STY FDCSECT
           JSR LCLRSTAT
           JSR LCHKPRT
           LDX #$01
           STX LNBIRQ
;
; Reset stack.
; Get ready for the next command from the computer.
;
LRESSTK    LDX #$FF
           TXS
;
; Go to track 0 ($D8 is
; $27 inverted to step back to track 0).
;
           LDA #$FF-$27
           JSR LTRKDIF
           STY LTRDIF
;
; Restore head to track 0.
;
LRESTORE   LDA #$FF-FDCREST
           STA FDCCOMND
           LDA #AUTTRK0
;
; Jump over sector table
;
           JMP LOVERTBL
;
; Spare bytes never addressed
;
      :25  .BYTE 0
;
; 18 inverted sector numbers
;
LFMTTAB    .BYTE $00,$FE,$FC,$FA
           .BYTE $F8,$F6,$F4,$F2
           .BYTE $F0,$EE,$FD,$FB
           .BYTE $F9,$F7,$F5,$F3
           .BYTE $F1,$EF
;
; Init continues here with the main loop waiting for SIO commands
;
LOVERTBL   JSR LORAUT
LMAINLP    BIT RIOTDRA
           BVS LHASIRQ0
           JSR LHASCMD
           JMP LMAINLP
LHASIRQ0   DEC LNBIRQ
           BPL LRESTORE
LRESET     LDX #$80
           LDY LTRDIF
           BEQ LRSTPORT
           JSR LSTEPTRK
           STY LTRDIF
           JSR LHASCMD
;
; Wait until the timer interrupt flag is set
;
LWSTIM     LDA RIOTRINT
           BPL LWSTIM
           BMI LRESET
;
; Reset Ports A & B and automata state
;
LRSTPORT   LDA #UNUSED
           STA RIOTDRA
           LDA #0
           STA RIOTDRB
           LDA #$FF-AUTTRK0
           JSR LANDAUT
           LDA #AUTWAIT
           JSR LORAUT
           JMP LHASCMD
;
; Check if sector number in AUX1/AUX2 is valid ($1 to $2D0).
;
LCHKSEC    LDX LCMDAUX2
           LDA LCMDAUX1
           BEQ LSECLOW
;
; Check high limit.
;
LSECHIGH   CPX #$03
           BCS LSECERR
           CMP #$D1
           BCS LSECLAST
;
; Sector number is valid
;
LSECOK     CLC
           RTS
;
; Check low limit.
;
LSECLOW    CPX #0
           BNE LSECHIGH
;
; Sector number is out of range.
;
LSECERR    SEC
           RTS
;
; If AUX1 is higher than $D0 then AUX2 should not exceed $1.
;
LSECLAST   CPX #$02
           BEQ LSECERR
           JMP LSECOK
           RTS
;
; Sleep for a certain amount of time.
;
LSLEEP     STA LPAUSE
LWAIT      DEY
           BNE LWAIT
           DEC LPAUSE
           BNE LWAIT
           RTS
;
; Use stepping motor to go to the previous (if X is negative) or the next (if X is positive) track.
;
LSTEPTRK   LDA #$29
           STA RIOTW64
           LDA RIOTDRB
           AND #PHASE1+PHASE2+PHASE3+PHASE4
           INX
           BMI LPREVTRK
;
; Go to next track
;
           INC LCURTRK
           LSR
           STA LCURDRB
           AND #PHASE4/2
           BEQ LSAVCUR
;
; report PHASE4 bit into PHASE1
;
           LDA #PHASE1
           ORA LCURDRB
           BPL LENDSTP
;
; Go to previous track
;
LPREVTRK   DEC LCURTRK
           ASL
           STA LCURDRB
           AND #PHASE1*2
           BEQ LSAVCUR
;
; report PHASE1 bit into PHASE4
;
           LDA #PHASE4
           ORA LCURDRB
           BPL LENDSTP
LSAVCUR    LDA LCURDRB
LENDSTP    STA RIOTDRB
           DEY
           RTS
;
; GET SECTOR
; ----------
;
LGET       JSR LCHKSEC
           BCS LBADSEC1
;
; Wait for SIO COMMAND to be clear and send ACK to the computer.
;
           JSR LCMDACK
;
; Seek to track containing sector
;
           JSR LMOTORON
           JSR LCHKWAIT
           JSR LSEEKTRK
;
; Issue a read sector command to the FDC
;
LRDSEC     LDA #$FF-FDCRSEC
           STA FDCCOMND
;
; Check if a FDC interrupt has been raised
;
LHASIRQ4   BVS LIRQDET
;
; Reset timer with interrupt
;
           LDA RIOTRTIT
;
; Read all sector bytes from the FDC
;
LWBYTE     BIT RIOTDRA
           BPL LHASIRQ4
           LDA FDCDATA
           INX
           STA SECTOR,X
           BPL LWBYTE
;
; Check the result of the read operation.
;
LIRQDET    JSR LRDSTAT
           BCS LTRYSEC
;
; Notify computer that the operation has been done.
;
LISCOMP    LDA #COMPLETE
LSNDRES    STA LSNDBYTE
           JSR LCHKDATA
           STA LCRCBYTE
;
; Send sector to the computer.
;
           JSR LDATAOUT
           JSR LWRSEC
           JMP LRESSTK
LTRYSEC    BNE LRDSEC
;
; Notify computer that an error occurred.
;
LHASERR    LDA #ERROR
           JMP LSNDRES
LBADSEC1   JMP LBADCMD
;
; This byte seems to be never addressed
;
           .BYTE $AD
;
; Read FDC status
;
LRDSTAT    LDA FDCSTAT
           STA LSTATHW
           EOR #$FF
           BNE LCKRNF
;
; Clear wait flag
;
           CLC
           LDA #$FF-AUTWAIT
           JSR LANDSTAT
LSETTK     JSR LSETTRK
           DEY
           RTS
;
; An error has been detected.
; Check if error is Record Not Found.
;
LCKRNF     CPY #$02
           BNE LSETERR
           AND #CTRLRNF
           BEQ LSETERR
           STY LPAUSE
           JSR LCHKWAIT
;
; The sector has not been found.
; Try to move the gead in case it was not over the right track
;
           LDY #$2B
           JSR LMOVHEAD
           LDA LITRKNUM
           JSR LTRKDIF
           JSR LMOVTRK
           LDY LPAUSE
;
; Set STATERR flag
;
LSETERR    LDA #STATERR
           JSR LORSTAT
           SEC
           JMP LSETTK
;
; FORMAT
; ------
;
LFORMAT    JSR LCMDACK
           JSR LMOTORON
           JSR LCHKWAIT
           LDY #$2B
           JSR LMOVHEAD
           JSR LWFINT
;
; Check Write Protect flag.
;
           LDA RIOTDRA
           AND #WPT1771
           BEQ LNWPROT
;
; Send an error if disk is write protected.
;
           LDA #STATWP
           JSR LORSTAT
           JMP LSTOPF
;
; Format 40 tracks.
; Location LITRACK will count from $FF to $D7 (inverted track number).
;
LNWPROT    LDA #$FF
           STA LITRACK
           JSR LSKIPSTP
LOOPTRK    DEC LITRACK
           LDA LITRACK
           CMP #$D7
           BEQ LENDTRK
           LDX #$80
           JSR LNXTTRK
           JMP LOOPTRK
LENDTRK    JMP LENDFMT
           BCS LNXTTRK
;
; Format one track
;
LNXTTRK    JSR LNXTSTEP
;
; Load pointer to standard sector list
;
LSKIPSTP   LDA #<LFMTTAB
           STA LSECPTR
           LDA #>LFMTTAB
           STA LSECPTR+1
;
; Read timer to disable interrupt
;
           LDA RIOTRTNO
;
; Location LISECTOR will count from $ED to $FF (inverted number of sectors to write).
;
           LDA #$ED
           STA LISECTOR
;
; Send the write track command to the FDC.
;
           LDY #$FF
           LDX #$FF
           LDA #$FF-FDCWTRK
           STA FDCCOMND
;
; Write a first $00 value ($FF for inverted).
;
           JSR LFDCWRT
;
; Then, write 255 bytes with a $00 value ($FF for inverted).
;
           LDA #$0D
           STA RIOTW1K
           LDY #$FF
           JSR LFDCWBUF
           STX RIOTW1K
;
; Write Index Mark $FC ($03 for inverted).
;
           LDY #$FF-FDCIDXAM
           JSR LFDCWRT
;
; Write 11 bytes with $00 value ($FF for inverted) from X register.
;
           LDY #$0B
           JSR LFDCWBUF
;
; Write 6 bytes with $00 value ($FF for inverted) from X register.
;
LNXTSEC    LDY #$06
           JSR LFDCWBUF
;
; Write $FE ($01 for inverted).
;
           LDY #$FF-FDCIDAM
           JSR LFDCWRT
;
; Write track number.
; It is already inverted.
;
           LDY LITRACK
           JSR LFDCWRT
;
; Write the side number which is always $00 ($FF for inverted).
;
           LDY #$FF
           JSR LFDCWRT
;
; Write sector number.
; It is already inverted.
;
           LDY LISECTOR
           JSR LFDCWRT
;
; Write the sector size which is always $00 ($FF for inverted) for 128 byte length.
;
           LDY #$FF
           JSR LFDCWRT
;
; Now a $F7 value is sent to the FDC.
; This is a special value used to generate 2 CRC bytes.
;
           LDY #$FF-FDCCRC
           JSR LFDCWRT
;
; Write 17 bytes with $00 value ($FF for inverted) from X register.
;
           LDY #$11
           JSR LFDCWBUF
;
; Write Data Address Mark $FB ($04 for inverted).
;
           LDY #$FF-FDCDATAM
           JSR LFDCWRT
;
; Write 128 bytes with $00 value which means the sector is blank filled.
;
           LDY #$80
           LDX #0
           JSR LFDCWBUF
;
; Now a $F7 value is sent to the FDC. This is a special value used to generate 2 CRC bytes.
;
           LDY #$FF-FDCCRC
           JSR LFDCWRT
;
; Write 9 bytes with $00 value ($FF for inverted) from X register.
;
           LDY #$09
           LDX #$FF
           JSR LFDCWBUF
;
; Write another byte with $00 value ($FF for inverted).
;
           JSR LFDCWRT
;
; Set next sector
;
           INC LSECPTR
           BMI LLASTSEC
;
; Write another byte with $00 value ($FF for inverted).
;
           JSR LFDCWRT
;
; Get next inverted sector number from the table.
;
           LDA (LSECPTR+1,X)
           STA LISECTOR
           JMP LFDCWR2
;
; If FDC triggers an IRQ, the formatting is stopped and an error is sent to the computer.
;
LHASIRQ3   BVS LSTOPF
;
; Write the last byte with $00 value ($FF for inverted).
;
LFDCWR2    BIT RIOTDRA
           BPL LHASIRQ3
           STY FDCDATA
           JMP LNXTSEC
;
; Write the last byte
;
LLASTSEC   LDA #$FF-FDCFINT
           STA FDCCOMND
           RTS
;
; If FDC triggers an IRQ, the formatting is stopped and an error is sent to the computer.
;
LHASIRQ2   BVS LSTOPF
;
; Write repeatedly the same byte to the FDC.
; Wait for FDC to be ready to receive a data byte, then the data byte in X register is written to the FDC.
;
LFDCWBUF   BIT RIOTDRA
           BPL LHASIRQ2
           STX FDCDATA
           DEY
           BNE LFDCWBUF
           RTS
;
; If FDC triggers an IRQ, the formatting is stopped and an error is sent to the computer.
;
LHASIRQ1   BVS LSTOPF
;
; Wait for FDC to be ready to receive a data byte. The DRQ1771 is checked.
; Then the data byte in Y register is written to the FDC.
;
LFDCWRT    BIT RIOTDRA
           BPL LHASIRQ1
           STY FDCDATA
           RTS
;
; Read FDC status and send an error to the computer.
;
LSTOPF     LDX #$FF
           STX SECTOR
           STX SECTOR+1
           LDA FDCSTAT
           STA LSTATHW
           JMP LRAISERR
;
; The 40 tracks have been formatted.
;
LENDFMT    LDX #0
           STX LOFSRES
;
; Read sector 1 of the current track to check if disk has been successfully formatted
;
LCHNGTK    LDA LCURTRK
           STA FDCTRACK
;
; Set the sector number in FDC.
; Start with sector 1
;
           LDA #$FF-$01
LNXSECT    STA LISECNUM
           STA FDCSECT
           LDY #$02
           STY LRETRIES
;
; Read next sector
;
LNXRDSC    JSR LRDSECT
           LDY #$01
           JSR LRDSTAT
           BCS LERFRMT
;
; Check another sector by adding 4 to the sector number.
; The sectors are read in this order:
; $01, $05, $09, $0D, $11, $04, $08, $0C, $10, $03, $07, $0B, $0F, $02, $06, $0A, $0E, $12
;
LCPTSEC    LDA LISECNUM
           SEC
           SBC #$04
           CMP #$FF-$15
           BNE LNLAST1
           LDA #$FF-$04
LNLAST1    CMP #$FF-$14
           BNE LNLAST2
           LDA #$FF-$03
LNLAST2    CMP #$FF-$13
           BNE LNLAST3
           LDA #$FF-$02
LNLAST3    CMP #$FF-$16
           BNE LNXSECT
;
; All sectors of this track have been read.
; Check if we go to the next track to read sectors.
;
           LDA LCURTRK
           CMP #$FF-$00
           BEQ LNOMORE
;
; Step forward to the next track
;
           LDX #0
           LDY #$01
           JSR LMOVTRK
           JMP LCHNGTK
LNOMORE    JMP LSAVRES1
;
; An error has been detected in the format operation when reading some sectors
; Do we try again ?
;
LERFRMT    DEC LRETRIES
           BNE LNXRDSC
           LDX LOFSRES
           JMP LSAVRES3
;
; Get DRVMOTOR bit from Port A.
;
LDRVBIT    LDA RIOTDRA
           LSR
           LSR
;
; Set the STATACT bit (active drive) in the command status byte if the DRVMOTOR bit is set.
;
           LDA #STATACT
           BCS LORSTAT
           LDA #STATNONE
;
; Modify command status byte.
;
LORSTAT    ORA LSTATCMD
           STA LSTATCMD
           RTS
;
; Set command status byte by clearing it and ORing it with the Accumulator.
;
LSETSTAT   PHA
           LDA #0
           AND LSTATCMD
           STA LSTATCMD
           PLA
           JMP LORSTAT
           RTS
;
; Add some bits to the automata
;
LORAUT     ORA LAUTOM
           STA LAUTOM
           RTS
;
; Remove some bits from the automata
;
LANDAUT    AND LAUTOM
           STA LAUTOM
           RTS
;
; Clear all status bits except Write Protect flag.
;
LCLRSTAT   LDA #STATWP
;
; Clear some status bits.
;
LANDSTAT   AND LSTATCMD
           STA LSTATCMD
           RTS
;
; Read 128 bytes from SIO.
; Wait for start bit.
;
LSTABIT    LDA RIOTDRB
           BPL LSTABIT
           TXA
           BEQ LWFIRST
;
; Invert and store byte.
;
           LDA LCMDDRV-1,X
           EOR #$FF
           STA LCMDDRV-1,X
LLOOPBT    NOP
           NOP
           NOP
;
; Read all 8 bits from SIO.
;
           LDY #$78
LNBIT      NOP
           NOP
           NOP
           NOP
           LDA RIOTDRB
           ROL
           ROR LCMDDRV,X
           INY
           BPL LNBIT
;
; Next byte
;
           INX
           CPX LENDCMD
           BNE LSTABIT
;
; Invert and store last byte and return.
;
           LDA LCMDDRV-1,X
           EOR #$FF
           STA LCMDDRV-1,X
           RTS
;
; Waste cycles before reading first bit.
;
LWFIRST    NOP
           NOP
           NOP
           JMP LLOOPBT
LNXTSTEP   LDA #$02
           JSR LSLEEP
           JSR LSTEPTRK
           RTS
;
; Receive a sector (128 bytes) from computer.
; The byte is transmitted bit after bit through the DATAIN line of port B.
;
LRCVSEC    LDX #0
           LDY #$07
           STY LENDCMD
;
; Wait for the start bit.
;
LWSTART    LDA RIOTDRB
           BPL LWSTART
;
; Do not invert byte in the first iteration because we did not read any byte yet.
;
           TXA
           BEQ LRFIRST
;
; Invert all bits of the byte that has been received.
;
           LDA SECTOR-1,X
           EOR #$FF
           STA SECTOR-1,X
;
; Loop to receive all bytes.
;
LRCVBYTE   NOP
           NOP
           NOP
           LDY #$78
;
; Loop to receive all bits of a byte.
;
LRCVBIT    NOP
           NOP
           NOP
           LDA SECTOR
;
; Get DATAIN line of port B and save it into SECTOR buffer.
;
           LDA RIOTDRB
           ROL
           ROR SECTOR,X
;
; Next bit...
;
           INY
           BPL LRCVBIT
;
; Next byte...
;
           INX
           BPL LRCVNEXT
;
; All bytes have been received.
;
           LDX #$06
           JSR LSTABIT
           LDA SECTOR+$78,X
           EOR #$FF
           STA SECTOR+$78,X
           RTS
;
; One NOP for timing and get ready for next byte.
;
LRCVNEXT   NOP
           JMP LWSTART
;
; Spend time to wait for the end of the start bit transmission.
;
LRFIRST    NOP
           NOP
           NOP
           JMP LRCVBYTE
;
; Compute a checksum of a command.
; Upon entry,
; - X contains the offset of the command from the label LCMDDRV.
; - Address LENDCMD contains the last offset of the command from the label LCMDDRV.
; This subroutine returns the checksum in A.
;
LCHKSUM    LDA #0
           CLC
LCHKLOOP   ADC LCMDDRV,X
           PHP
;
; Is it the last byte of the command ?
;
           INX
           CPX LENDCMD
;
; Last byte has been processed.
; Get out of the loop.
;
           BEQ LCHKEND
           PLP
           JMP LCHKLOOP
;
; The checksum is computed.
; We add the carry.
;
LCHKEND    PLP
           ADC #0
;
; Compare checksum with the one from the command.
; This is useful only with the command received from the computer.
;
           CMP LCMDDRV,X
           RTS
;
; Absolute sector number given in DAUX1/DAUX2 is converted into a track number and a sector number.
;
LCNVSEC    LDA LCMDAUX2
           STA LSAVAUX2
           LDA LCMDAUX1
           STA LSAVAUX1
;
; Sector number is divided by 16.
;
           LDX #$04
LCNVDIV    ROR LCMDAUX2
           ROR
           DEX
           BNE LCNVDIV
           ROR LCMDAUX2
;
; LCMDAUX2 now contains in its 4 high bits the value of DAUX1 4 low bits.
; LCMDAUX2 is shifted right 4 times to have a value between $01 and $0F.
;
           LDX #$04
LCNVHI     LSR LCMDAUX2
           DEX
           BNE LCNVHI
;
; A contains absolute sector number divided by 16.
; A is multiplied by 2 to be compared to LCMDAUX2.
;
LCNVEND    ASL
;
; If C is Clear, it means that A contains the track number multiplied by 2.
; If C is Set, it means that LCMDAUX2 should be adjusted to get the track number.
;
           CMP LCMDAUX2
           BCS LCNVSET
;
; A is divided by 2 to get the track number.
; It is saved inverted.
;
           STA LCMDAUX1
           LSR
           EOR #$FF
           STA LITRKNUM
           SEC
           LDA LCMDAUX2
           SBC LCMDAUX1
           EOR #$FF
           STA LISECNUM
           RTS
;
; The division of LCMDAUX2 by 16 has to be adjusted to get the track number.
;
LCNVSET    LSR
           TAX
           DEX
           LDA #$10
           CLC
           ADC LCMDAUX2
           STA LCMDAUX2
           TXA
           JMP LCNVEND
;
; Compute the distance between the current track and the one to seek to.
; Y will hold the difference
; X will hold the direction
;
LTRKDIF    LDX #0
           LDY #0
           SEC
           SBC LCURTRK
           BEQ LZERO
           BPL LPOSITIV
           LDX #$80
           EOR #$FF
           CLC
           ADC #$01
LPOSITIV   TAY
LZERO      RTS
;
; Check if disk is write protected and store the info in status byte.
;
LCHKPRT    LDA RIOTDRA
           AND #WPT1771
           BEQ LNOPRT
           LDA #STATWP
           JSR LORSTAT
           RTS
;
; The disk is not write protected
;
LNOPRT     LDA #$FF-STATWP
           JSR LANDSTAT
           RTS
;
; Start drive motor (if not already started).
;
LMOTORON   LDA RIOTDRA
;
; Check DRVMOTOR bit.
; If bit is set, it's OK.
;
           LSR
           LSR
           BCS LDRVISON
;
; Start drive motor now.
;
           LDA #DRVMOTOR+UNUSED
           STA RIOTDRA
           STA FDCDATA
           LDA #$16
           STA FDCTRACK
;
; Send the seek command to the FDC
;
           LDA #$FF-FDCSEEK
           STA FDCCOMND
LDRVISON   LDA #$01
           STA LNBIRQ
           RTS
;
; Check if we have to wait for a previous operation (bit AUTWAIT of LAUTOM).
;
LCHKINT    LDA LAUTOM
           LSR
           LSR
           LSR
           BCC LACTIVE
;
; Wait for a FDC interrupt.
;
LFDCINT    BIT RIOTDRA
           BVC LFDCINT
           LDA #$FF-AUTWAIT
           JSR LANDAUT
LACTIVE    LDA #STATACT
           JSR LSETSTAT
           RTS
;
; Wait for the SIO COMMAND bit to be clear.
;
LCMDACK    BIT RIOTDRB
           BVS LCMDACK
;
; Send an ACK byte to the Atari
;
LSENDACK   LDA #ACK
;
; Send a byte from the Accumulator to the Atari.
;
LSEND      STA LSNDBYTE
           JSR LDATAOUT
           JSR LSNDDATA
           JSR LNODATA
           RTS
;
; start moving the head
;
LMOVHEAD   LDX #PHASE1+PHASE4
           STX RIOTDRB
LMOVNEXT   JSR LSTEPTRK
           BMI LMOVDONE
;
; Wait until the timer interrupt flag is set
;
LMOVWAIT   LDA RIOTRINT
           BPL LMOVWAIT
           JMP LMOVNEXT
LMOVDONE   STY LCURTRK
;
; wait some time...
;
           TXA
           LDX #$14
LMOVTIME   DEY
           BNE LMOVTIME
           DEX
           BNE LMOVTIME
           TAX
           RTS
;
; Check if Computer has raised the SIO command line.
;
LHASCMD    LDA RIOTDRB
           LSR
           LSR
           BCS LCMDUP
;
; Check the automata for the AUTLINE bit.
; It gives the previous state of the SIO command line.
;
           LDA LAUTOM
           LSR
           BCS LHADLINE
;
; Check the SIO command line
;
           BIT RIOTDRB
           BVC LINEDOWN
;
; The SIO command line is up.
; Save the new state in the automata if not already done
;
           BIT LAUTOM
           BVC LSETCMD
;
; Wait a little bit
;
           LDA #$29
           JSR LSLEEP
;
; Check again the SIO command line
;
           BIT RIOTDRB
           BVS LSETCMD
;
; SIO command line is down
;
LINEDOWN   LDA LAUTOM
           AND #AUT40+AUTTRK0
           BNE LNOCHK
           JMP LHASCMD
LNOCHK     RTS
;
; The AUTLINE of the automata was up.
; Wait a little bit and reset the flag
;
LHADLINE   LDA #$29
           JSR LSLEEP
           LDA #$FF-AUTLINE
           JSR LANDAUT
           JMP LHASCMD
;
; Command line is up
;
LCMDUP     LDA #AUTLINE
           JSR LORAUT
           JMP LINEDOWN
;
; The state of the SIO command line is saved in the automata
;
LSETCMD    LDA #AUTCMD
           JSR LORAUT
           LDX #0
           LDA #$05
           STA LENDCMD
           JSR LSTABIT
           LDX #0
           DEC LENDCMD
           JSR LCHKSUM
           BEQ LREADCFG
           LDA #AUTBAD
           JSR LORAUT
;
; Get drive number from the switches at the rear of the drive.
; This is done by reading 2 bits from Port A of the 6532.
; Here are the possible values:
;
; SWITCH1 SWITCH2 Drive
;   ON      ON      1
;   OFF     ON      2
;   OFF     OFF     3
;   ON      OFF     4
;
; X will hold the drive number (by default 4):
;
LREADCFG   LDX #$34
;
; Read port A and keep only bits from the 2 switches.
;
           LDA RIOTDRA
           AND #SWITCH1+SWITCH2
;
; Switch configuration:
; SWITCH1 SWITCH2 Drive
;   OFF     OFF     3
;
           BEQ LDRV3
; 
; Switch configuration:
; SWITCH1 SWITCH2 Drive
;   OFF     ON      2
;
           CMP #SWITCH2
           BEQ LDRV2
;
; Switch configuration:
; SWITCH1 SWITCH2 Drive
;   ON      OFF     4
;
           CMP #SWITCH1
           BEQ LDRV4
;
; Switch configuration:
; SWITCH1 SWITCH2 Drive
;   ON      ON      1
;
           DEX
LDRV2      DEX
LDRV3      DEX
;
; X now holds the drive number ($31-$34) from the switches.
; Check that the command is for our drive.
;
LDRV4      CPX LCMDDRV
           BEQ LDRVOK
;
; Command is not for our drive.
; Wait for the SIO COMMAND bit to be clear.
;
LCMDSET1   BIT RIOTDRB
           BVS LCMDSET1
           LDA #$FF-AUTCMD-AUTBAD
           JSR LANDAUT
           JMP LHASCMD
;
; Check command sent by computer.
;
LDRVOK     JSR LCHKCMD
           JMP LHASCMD
;
; GET STATUS
; ----------
;
LSTATUS    JSR LSENDACK
;
; Compute checksum on the status frame answer.
;
           LDX #LSTATCMD-LCMDDRV
           LDY #LSTATCMD-LCMDDRV+STATLEN
           STY LENDCMD
           JSR LCHKSUM
;
; Store checksum at the end of the status frame answer.
;
           STA LSTATSUM
;
; Send a COMPLETE code to the computer.
;
           LDA #COMPLETE
           STA LSNDBYTE
           JSR LDATAOUT
;
; X will hold the count of bytes to send.
; The loop ends when reg X reaches $80. X is set to $80 minus the COMPLETE byte, minus the 4 bytes
; of the status answer and minus the checksum byte.
;
           LDX #$80-1-STATLEN-SUMLEN
           JSR LSNDDATA
;
; Send all status bytes to the computer.
;
LSTATLP    LDY LSTATCMD-$80+STATLEN+SUMLEN,X
           STY LSNDBYTE
           JSR LSNDDAT2
           BPL LSTATLP
;
; All bytes have been sent.
; Turn off DATA OUT line.
;
           JSR LNODATA
;
; Reset all status bits except Write Protect then set Activity bit according to the WD1771 status.
;
           JSR LCLRSTAT
           JSR LDRVBIT
           JMP LRESSTK
;
; Convert the track and relative sector found in FDC registers into an absolute sector number.
;
LABSSECT   LDY #0
           STY LABSECHI
           LDA LISECNUM
           EOR #$FF
           STA LSECNUM
           LDA LCURTRK
           EOR #$FF
           ASL
           STA LTRKTEMP
           ASL
           ASL
           ROL LABSECHI
           ASL
           ROL LABSECHI
           ADC LTRKTEMP
           TAY
           LDA #0
           ADC LABSECHI
           JMP LSAVRES2
;
; Check AUTBAD flag in automata state.
;
LCHKCMD    LDA LAUTOM
           LSR
           LSR
           BCS LBADCMD
;
; The command should be in the range of $50 to $57.
;
           LDA LCMDCMD
           CMP #$4F
           BCC LISFMT
           CMP #$58
           BCS LISFMT
;
; Command is converted to a Jump address.
; The 3 low bits of the command byte are multiplied by 3.
;
           AND #$07
           STA LOFSCMD
           ASL
           ADC LOFSCMD
;
; Then, the result is added to LTABCMD address.
;
           ADC #<LTABCMD
           STA LJMPCMD
           LDA #0
           ADC #>LTABCMD
           STA LJMPCMD+1
;
; Execute the routine corresponding to the command.
;
           JMP (LJMPCMD)
;
; Is command equal to $21 (format).
;
LISFMT     CMP #$21
           BEQ LJMPFMT
;
; The command sent is unknown !
;
LBADCMD    LDA #STATCMD
           JSR LSETSTAT
           JSR LDRVBIT
;
; Wait for the SIO COMMAND bit to be clear.
;
LCMDSET2   BIT RIOTDRB
           BVS LCMDSET2
;
; Send NACK to the Atari.
;
           LDA #NACK
           JSR LSEND
           LDA #$FF-AUTBAD
           JSR LANDAUT
           RTS
;
; Jump to format routine
;
LJMPFMT    JMP LFORMAT
;
; It seems that this byte is never addressed
;
           .BYTE $01
;
; Save result of the format command
;
LSAVRES2   STA LABSECHI
           TYA
           CLC
           ADC LSECNUM
           STA SECTOR,X
           LDA #0
           ADC LABSECHI
           INX
           STA SECTOR,X
           RTS
;
; Compute data checksum and check it.
;
LCHKDATA   LDA #0
           TAX
           CLC
LADDBYTE   ADC SECTOR,X
           INX
           BPL LADDBYTE
           ADC #0
           CMP LCRCBYTE
           RTS
;
; Set DATAOUT line of port B as an output line.
;
LDATAOUT   LDX RIOTDRB
           STX RIOTDRB
           LDX RIOTDDRB
           INX
           STX RIOTDDRB
           RTS
;
; Send a sector to the Atari
;
LWRSEC     LDX #$FF
           JSR LSNDDATA
LWRLOOP    LDY SECTOR,X
           STY LSNDBYTE
           JSR LSNDDAT2
           BPL LWRLOOP
           LDY LCRCBYTE
           STY LSNDBYTE
           JSR LSNDDAT2
;
; Set DATAOUT line of Port B as an input line: No more data is sent to the Atari.
;
LNODATA    LDX RIOTDDRB
           DEX
           STX RIOTDDRB
           RTS
;
; Send a byte to the Atari.
; Each bit of the byte is put on the DATAOUT line of Port B (LSB bit first).
;
LSNDDATA   LDA RIOTDRB
;
; A start bit (value 0) is sent before the byte.
;
LSNDDAT2   AND #$FE
           STA RIOTDRB
;
; Y register will hold the bit counter (from $78 to $80).
;
           LDY #$78
           BIT SECTOR
LSNDLOOP   BIT SECTOR
           NOP
           NOP
;
; Send one bit from LSNDBYTE.
;
           LSR
           ROR LSNDBYTE
           ROL
           STA RIOTDRB
           INY
           BPL LSNDLOOP
;
; A stop bit (value 1) is sent after the byte.
;
           LSR
           SEC
           ROL
           NOP
           NOP
           NOP
           NOP
           NOP
           NOP
           STA RIOTDRB
           INX
           RTS
;
; Save result of the format command
;
LSAVRES1   LDA #$FF
           LDX LOFSRES
           STA SECTOR,X
           INX
           STA SECTOR,X
;
; Is it a write with verify ?
;
           LDA LAUTOM
           AND #AUTVER
           BNE LRSTVER
           JMP LISCOMP
;
; Reset the verify flag before sending an error
;
LRSTVER    LDA #$FF-AUTVER
           JSR LANDAUT
;
; Set error in status and send an error to the Atari
;
LRAISERR   LDA #STATERR
           JSR LORSTAT
           JMP LHASERR
;
; Get absolute sector which triggers an error and save its number in output buffer
;
LSAVRES3   LDA #AUTVER
           JSR LORAUT
           JSR LABSSECT
           INX
           STX LOFSRES
           CPX #$7E
           BNE LMORERES
           JMP LSAVRES1
LMORERES   JMP LCPTSEC
;
; This is the jump table for the drive commands.
; The table is the following:
;
; Command Meaning
;   $50   PUT SECTOR
;   $51   UNUSED
;   $52   GET SECTOR
;   $53   GET STATUS
;   $54   UNUSED
;   $55   UNUSED
;   $56   UNUSED
;   $57   PUT SECTOR VERIFY
;
LTABCMD    JMP LPUT
           JMP LBADCMD
           JMP LGET
           JMP LSTATUS
           JMP LBADCMD
           JMP LBADCMD
           JMP LBADCMD
           JMP LPUTVER
;
; Wait for SIO COMMAND to be clear and send ACK to computer.
;
LDOPUT     JSR LCMDACK
           JSR LMOTORON
;
; Receive the sector from the computer
;
           JSR LRCVSEC
           JSR LCHKDATA
           BNE LBADDATA
;
; Everything has been received.
;
           JSR LSENDACK
           JSR LCHKWAIT
           JSR LSEEKTRK
;
; Check Write Protect flag.
;
           LDA RIOTDRA
           AND #WPT1771
           BNE LWPROT
;
; The disk is not write protected.
; Send the write sector to the FDC
;
LWRSECT    LDA #$FF-FDCWSEC
           STA FDCCOMND
;
; Check if a FDC interrupt has been raised
;
LHASIRQ7   BVS LRDERR
;
; Reset timer with interrupt
;
           LDA RIOTRTIT
;
; Wait until the FDC has a byte from the sector
;
           BIT RIOTDRA
           BMI LHASBYTE
           JMP LHASIRQ7
;
; Check if a FDC interrupt has been raised
;
LHASIRQ8   BVS LRDERR
;
; Wait until the FDC has a byte from the sector
;
LNXBYTE    BIT RIOTDRA
           BPL LHASIRQ8
;
; Send next byte to the FDC
;
LHASBYTE   INX
           LDA SECTOR,X
           STA FDCDATA
           JMP LNXBYTE
;
; A FDC interrupt has been detected.
;
LRDERR     JSR LRDSTAT
           LDA #ERROR
           BCS L0F42
;
; Start verify the data from the disk if a write with verify has been requested
;
           LDA LAUTOM
           AND #AUTVER
           BEQ LNOVER
           JSR LVERIFY
           BCC LNOVER
           LDA #ERROR
           BCS LSENDRES
LNOVER     LDA #COMPLETE
LGORES     JMP LSENDRES
;
; The read verification after the write has detected an error.
; Send an error to the computer
;
LBADDATA   LDA #STATACT+STATDATA
           JSR LSETSTAT
           LDA #NACK
LSENDRES   JSR LSEND
           JMP LRESSTK
;
; A write has been issued on a write protected disk.
; Send an error to the Atari
;
LWPROT     LDA #STATERR+STATWP
           JSR LORSTAT
           LDA #ERROR
           JMP LSENDRES
;
; Move to the destination track
;
LMOVTRK    BEQ LENDTM
LNXTMOVE   JSR LSTEPTRK
           BEQ LWAITM
;
; Wait until the timer interrupt flag is set
;
LOOPTM2    LDA RIOTRINT
           BPL LOOPTM2
           JMP LNXTMOVE
;
; Waste some time to reach the desired track
;
LWAITM     LDA #$50
           STA RIOTW64
;
; Wait until the timer interrupt flag is set
;
LOOPTM1    LDA RIOTRINT
           BPL LOOPTM1
LENDTM     RTS
;
; PUT SECTOR with VERIFY
; ----------------------
;
LPUTVER    LDA #AUTVER
           JSR LORAUT
;
; PUT SECTOR
; ----------
;
LPUT       JSR LCHKSEC
           BCS LBADSEC2
           JMP LDOPUT
LBADSEC2   JMP LBADCMD
L0F42      CPY #0
           BEQ L0F49
           JMP LWRSECT
L0F49      BEQ LGORES
;
; Convert absolute sector number to a track and a sector number then seek to the track.
;
LSEEKTRK   JSR LCNVSEC
           LDA RIOTDRB
           AND #PHASE1+PHASE2+PHASE3+PHASE4
           BNE LHASPH
           LDA #PHASE1+PHASE2
           STA RIOTDRB
LHASPH     LDA LITRKNUM
           JSR LTRKDIF
           JSR LMOVTRK
LWFINT     JSR LCHKINT
;
; Force interrupt
;
           LDY #$04
LSETTRK    LDA #$FF-FDCFINT
           STA FDCCOMND
;
; Wait while force interrupt is executed
;
           LDX #$FF
LWASTETM   DEX
           BNE LWASTETM
;
; Set track and sector numbers in FDC
;
           LDA LITRKNUM
           STA FDCTRACK
           LDA LISECNUM
           STA FDCSECT
           CLV
           DEX
           STX RIOTW1K
           RTS
;
; Read a sector
;
LRDSECT    JSR LSETTRK
           LDA LCURTRK
           STA FDCTRACK
;
; Send the read sector command to the  FDC
;
           LDA #$FF-FDCRSEC
           STA FDCCOMND
;
; Check if a FDC interrupt has been raised
;
LHASIRQ5   BVS LENDREAD
;
; Reset timer with interrupt
;
           LDA RIOTRTIT
;
; Wait until the FDC has a byte from the sector
;
LGETBYTE   BIT RIOTDRA
           BPL LHASIRQ5
;
; Read the byte from the FDC.
; Bytes are ignored because this is just to check if the sector can be read after format command
;
           LDA FDCDATA
           JMP LGETBYTE
LENDREAD   RTS
;
; Verify that the write operation is OK by reading the sector and comparing it with the sector buffer
;
LVERIFY    JSR LSETTRK
           LDY #$01
L0FA2      INX
;
; Send the read sector command to the FDC
;
           LDA #$FF-FDCRSEC
           STA FDCCOMND
;
; Check if a FDC interrupt has been raised
;
LHASIRQ6   BVS LIRQFND
;
; Reset timer with interrupt
;
           LDA RIOTRTIT
;
; Wait until the FDC has a byte from the sector
;
LCMPBYTE   BIT RIOTDRA
           BPL LHASIRQ6
;
; Read the byte and compare it with the data in the sector buffer
;
           LDA FDCDATA
           CMP SECTOR,X
           BNE LBYTEDIF
;
; Next byte...
;
           INX
           BNE LCMPBYTE
;
; A FC interrupt has been raised.
;
LIRQFND    JSR LRDSTAT
           BCC LUNSETV
           BPL L0FA2
;
; Remove the verify flag from the automata and end the operation
;
LUNSETV    LDA #$FF-AUTVER
           JSR LANDAUT
           RTS
;
; The byte is different. Send an error
;
LBYTEDIF   JSR LSETERR
           JMP LUNSETV
;
; Check the automata to see if we have to sleep.
;
LCHKWAIT   LDA LAUTOM
           AND #AUTWAIT
           BEQ LNOWAIT
           LDA #$50
           JSR LSLEEP
LNOWAIT    RTS
;
; This is just a padding so that the last two WORDs starts at address $0FFC and $0FFE.
;
      :34  .BYTE 0
;
; START is both the IRQ and RESET address.
;
; IMPORTANT: if you modify this source, always check that this word is at address $0FFC.
; If not, add or remove some of the padding bytes above.
;
           .WORD START ; RESET
           .WORD START ; IRQ

